/**
 * 
 */

package com.qihoo.appstore.updatelib;

import com.qihoo.appstore.updatelib.UpdateManager.CheckUpdateListener;

import org.json.JSONObject;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.pm.PackageInfo;
import android.net.Uri;
import android.os.Bundle;
import android.os.ResultReceiver;
import android.text.TextUtils;
import android.util.AndroidRuntimeException;
import android.util.Log;

import java.io.File;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * 更新服务
 * 
 * @author zhangyong6
 */
public class CheckUpdateService extends MultiIntentService {

    private static final String EXTRA_AUTO_DOWNLOAD = "AutoDownload";
    private static final String EXTRA_INSTALL_AND_OPEN_DETAIL = "InstallAndOpenDetail";
    private static final String EXTRA_INSTALL = "Install";
    private static final String EXTRA_INSTALL_AND_WAIT = "InstallAndWait";
    private static final String EXTRA_URL = "url";
    private static final String EXTRA_FILE = "file";
    static final String EXTRA_UPDATE_INFO = "UpdateInfo";
    private static final String EXTRA_AUTO_SHOW_DIALOG_OR_NOTIFICATION = "autoShowDialogOrNotification";
    private static final String EXTRA_CHANNEL = "channel";
    private static final String EXTRA_NOTIFICATION = "notification";
    private static final String EXTRA_RESULT_RECEIVER = "ResultReceiver";

    private static final String TAG = "CheckUpdateService";

    private static final String ACTION_DOWNLOAD = "com.qihoo.checkupdate.sdk.ACTION_DOWNLOAD";
    private static final String ACTION_CHECKUPDATE = "com.qihoo.checkupdate.sdk.ACTION_CHECKUPDATE";
    private static final String ACTION_START_APPSTORE = "com.qihoo.checkupdate.sdk.ACTION_START_APPSTORE";

    private AtomicBoolean sIsDownloading = new AtomicBoolean(false);

    public CheckUpdateService() {
        super("CheckUpdateService");
    }

    /**
     * @param name
     */
    public CheckUpdateService(String name) {
        super(name);
    }

    @Override
    protected void onHandleIntent(final Intent intent) {
        String action = intent.getAction();
        if (ACTION_START_APPSTORE.equals(action)) {
            doStartAppStore(intent);
        } else if (ACTION_DOWNLOAD.equals(action)) {
            doDownload(intent);
        } else if (ACTION_CHECKUPDATE.equals(action)) {
            if (!sIsDownloading.get()) {
                doCheckupdate(intent);
            } else {
                if (UpdateManager.isDebug()) {
                    Log.d(TAG, "onStart doCheckupdate downloading. skip");
                }
            }
        } else {
            if (UpdateManager.isDebug()) {
                Log.d(TAG, "onStart intent=" + intent.toUri(0) + ",fail,no action...");
            }
        }
    }

    static void startDownload(Context context, String url, String file, String title,
            int type, ResultReceiver resultReceiver, String detailPackageName) {
        Intent intent = new Intent(context, CheckUpdateService.class);
        intent.setAction(ACTION_DOWNLOAD);
        intent.putExtra(EXTRA_URL, url);
        intent.putExtra(EXTRA_FILE, file);
        intent.putExtra("title", title);
        if (type == 1) {
            intent.putExtra(EXTRA_INSTALL, true);
        } else if (type == 2) {
            // 下载，安装，并等待安装完成
            intent.putExtra(EXTRA_INSTALL_AND_WAIT, true);
        } else if (type == 3) {
            // 下载，安装，并等待安装完成，然后打开
            intent.putExtra(EXTRA_INSTALL_AND_OPEN_DETAIL, true);
        }
        if (!TextUtils.isEmpty(detailPackageName)) {
            intent.putExtra("DetailPackageName", detailPackageName);
        }
        intent.putExtra(EXTRA_RESULT_RECEIVER, resultReceiver);
        context.startService(intent);
    }

    static void checkUpdate(Context context, String packageName, String channel,
            boolean notification,
            ResultReceiver resultReceiver, boolean autoShowDialogOrNotification) {
        Intent intent = new Intent(context, CheckUpdateService.class);
        intent.setAction(ACTION_CHECKUPDATE);
        intent.putExtra(EXTRA_CHANNEL, channel);
        intent.putExtra(EXTRA_NOTIFICATION, notification);
        intent.putExtra(EXTRA_AUTO_SHOW_DIALOG_OR_NOTIFICATION, autoShowDialogOrNotification);
        if (resultReceiver != null) {
            intent.putExtra(EXTRA_RESULT_RECEIVER, resultReceiver);
        }
        intent.setData(Uri.parse("package:" + packageName));
        context.startService(intent);
    }

    static void startAppStoreToUpdate(Context context, String packageName, boolean autoDownload) {

        Intent intent = new Intent(context, CheckUpdateService.class);
        intent.setAction(ACTION_START_APPSTORE);
        intent.setData(Uri.parse("package:" + packageName));
        intent.putExtra(EXTRA_AUTO_DOWNLOAD, autoDownload);
        context.startService(intent);
    }

    private void doStartAppStore(Intent intent) {
        try {
            final Uri data = intent.getData();
            final String packageName = data != null ? data.getEncodedSchemeSpecificPart() : null;
            if (!TextUtils.isEmpty(packageName)) {
                Utils.startAppStoreToUpdate(this,
                        packageName, intent.getBooleanExtra(EXTRA_AUTO_DOWNLOAD, true));
            } else {
                if (UpdateManager.isDebug()) {
                    Log.e(TAG, "doStartAppStore packagename is null");
                }
            }
        } catch (Exception e) {
            if (UpdateManager.isDebug()) {
                Log.e(TAG, "doStartAppStore", e);
            }
        }
    }

    private synchronized void doCheckupdate(Intent intent) {
        ResultReceiver resultReceiver = null;
        if (intent != null) {
            resultReceiver = intent.getParcelableExtra(EXTRA_RESULT_RECEIVER);
        }
        int resultCode = UpdateManager.RESULT_FAIL;
        Bundle resultData = new Bundle();
        try {
            final Uri data = intent.getData();
            final String packageName = data != null ? data.getEncodedSchemeSpecificPart() : null;
            final boolean notification = intent.getBooleanExtra(EXTRA_NOTIFICATION, false);
            final String channel = intent.getStringExtra(EXTRA_CHANNEL);
            final boolean autoShowDialogOrNotification = intent.getBooleanExtra(
                    EXTRA_AUTO_SHOW_DIALOG_OR_NOTIFICATION, true);
            if (TextUtils.isEmpty(packageName)) {
                if (UpdateManager.isDebug()) {
                    Log.w(TAG, "doCheckupdate packageName is null,return");
                }
                resultData.putString(CheckUpdateListener.KEY_ERROR_MSG,
                        "doCheckupdate packageName is null,return");
                return;
            }

            if (TextUtils.isEmpty(channel)) {
                if (UpdateManager.isDebug()) {
                    Log.w(TAG, "doCheckupdate channel is null,return");
                }
                resultData.putString(CheckUpdateListener.KEY_ERROR_MSG, "channel is null,return");
                return;
            }

            String url = Utils.getCheckUpdateUrl(this, packageName,
                    channel);
            String strJson = HttpHelper.get(this, url);
            if (!TextUtils.isEmpty(strJson)) {
                AppInfo info = new AppInfo(new JSONObject(strJson));
                info.packageName = packageName;
                resultData.putParcelable(EXTRA_UPDATE_INFO, info);

                if (info.isNewVersion) {
                    if (autoShowDialogOrNotification) {
                        if (!notification) {
                            // 弹窗通知
                            Intent checkUpdateAcitivtyIntent = new Intent(this,
                                    CheckUpdateAcitivty.class);
                            checkUpdateAcitivtyIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                            checkUpdateAcitivtyIntent.putExtra(EXTRA_UPDATE_INFO, info);
                            startActivity(checkUpdateAcitivtyIntent);
                        } else {
                            // 显示通知栏通知
                            NotificationHelper.notifyNewUpdate(this, info);
                        }
                    }
                }
                resultCode = UpdateManager.RESULT_OK;
            } else {
                if (UpdateManager.isDebug()) {
                    Log.e(TAG, "doCheckupdate HTTP response is null");
                }
                resultData.putString(CheckUpdateListener.KEY_ERROR_MSG,
                        "HTTP response is null");
            }
        } catch (Throwable e) {
            if (UpdateManager.isDebug()) {
                Log.e(TAG, "doCheckupdate", e);
            }
            resultCode = UpdateManager.RESULT_FAIL;

            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            e.printStackTrace(pw);
            resultData.putString(CheckUpdateListener.KEY_ERROR_MSG, sw.toString());
            resultData.putString("Exception", e.getClass().getName());
            resultData.putString("ExceptionStackTrace", sw.toString());
        } finally {
            if (resultReceiver != null) {
                resultReceiver.send(resultCode, resultData);
            }
        }
    }

    private void doDownload(final Intent intent) {
        sIsDownloading.set(true);

        int resultCode = UpdateManager.RESULT_FAIL;
        final Bundle resultData = new Bundle();

        final String filepath = intent.getStringExtra(EXTRA_FILE);
        final ResultReceiver resultReceiver = intent.getParcelableExtra(EXTRA_RESULT_RECEIVER);
        if (TextUtils.isEmpty(filepath)) {
            resultData.putString(CheckUpdateListener.KEY_ERROR_MSG, "File path is null");
            return;
        }

        final File file = new File(filepath);
        final String url = intent.getStringExtra(EXTRA_URL);
        File temp = null;
        try {
            NotificationHelper.notifyDownloadProgress(CheckUpdateService.this,
                    intent.getStringExtra("title"), 0, 0);

            if (!TextUtils.isEmpty(url) && url.equals(Utils.getAppStoreDownloadUrl(this))) {
                temp = new File(Utils.getUpdateTempFile(this, HttpHelper.getReslUrl(this, url)));
            } else {
                temp = new File(Utils.getUpdateTempFile(this, url));
            }

            File dir = file.getParentFile();
            if (!dir.exists()) {
                dir.mkdirs();
            }
            if (file.exists()) {
                file.delete();
            }

            String downloadedPath = HttpHelper.get(this, url, temp,
                    new HttpHelper.ProgressListener() {

                        private int mLastProgress = -1;

                        @Override
                        public void onProgressBegin() {
                        }

                        @Override
                        public void onProgressChanged(long progress, long max) {
                            int per = (int) (100f * (float) progress / max);
                            if (mLastProgress != per) {
                                NotificationHelper.notifyDownloadProgress(CheckUpdateService.this,
                                        intent.getStringExtra("title"), progress, max);
                                mLastProgress = per;
                            }
                        }

                        @Override
                        public void onProgressEnd() {
                        }
                    });
            File downloadedFile = new File(downloadedPath);
            downloadedFile.renameTo(file);
            if (file.exists()) {
                Process process = Runtime.getRuntime().exec("chmod 755 " + file.getPath());
                process.waitFor();
                process = Runtime.getRuntime().exec("chmod 755 " + file.getParent());
                process.waitFor();

                if (intent.getBooleanExtra(EXTRA_INSTALL, false)) {
                    Utils.installApk(this, file.getPath());
                    resultCode = UpdateManager.RESULT_OK;
                } else if (intent.getBooleanExtra(EXTRA_INSTALL_AND_WAIT, false)) {
                    NotificationHelper.notifyDownloadComplete(this, intent.getStringExtra("title"),
                            file);
                    // 这里尽管任务还没完成，但是也还是可以继续让后面的任务执行吧。。否则，，否则。。。。
                    sIsDownloading.set(false);
                    resultCode = installAndWaitComplete(this, file.getPath(), 1000 * 60 * 4) ? UpdateManager.RESULT_OK
                            : UpdateManager.RESULT_FAIL;
                } else if (intent.getBooleanExtra(EXTRA_INSTALL_AND_OPEN_DETAIL, false)) {
                    // 这里尽管任务还没完成，但是也还是可以继续让后面的任务执行吧。。否则，，否则。。。。
                    sIsDownloading.set(false);
                    NotificationHelper.notifyDownloadComplete(this, intent.getStringExtra("title"),
                            file);
                    final String detailPackageName = intent.getStringExtra("DetailPackageName");
                    if (installAndWaitComplete(CheckUpdateService.this, file.getPath(),
                            1000 * 60 * 4)) {
                        Utils.startAppStoreToUpdate(CheckUpdateService.this,
                                detailPackageName, true);
                        resultCode = UpdateManager.RESULT_OK;
                    }
                } else {
                    resultCode = UpdateManager.RESULT_OK;
                }
            } else {
                resultCode = UpdateManager.RESULT_FAIL;
                resultData.putString(CheckUpdateListener.KEY_ERROR_MSG, "Dest file not exists");
            }
        } catch (Throwable e) {
            if (file != null) {
                file.delete();
            }

            if (UpdateManager.isDebug()) {
                Log.e(TAG, "doDownload", e);
            }

            if (intent != null
                    && (intent.getBooleanExtra(EXTRA_INSTALL_AND_WAIT, false) || intent
                            .getBooleanExtra(
                                    EXTRA_INSTALL_AND_OPEN_DETAIL, false))) {
                NotificationHelper.notifyDownloadComplete(this, intent.getStringExtra("title"),
                        file);
            } else {
                // DONOTHING
            }

            resultCode = UpdateManager.RESULT_FAIL;

            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            e.printStackTrace(pw);
            resultData.putString(CheckUpdateListener.KEY_ERROR_MSG, sw.toString());
            resultData.putString(CheckUpdateListener.KEY_EXCEPTION_CLASS_NAME, e.getClass()
                    .getName());
            resultData.putString(CheckUpdateListener.KEY_EXCEPTION_STACK_TRACE, sw.toString());
        } finally {
            if (resultReceiver != null) {
                resultReceiver.send(resultCode, resultData);
            }
            if (intent != null
                    && (intent.getBooleanExtra(EXTRA_INSTALL_AND_WAIT, false) || intent
                            .getBooleanExtra(
                                    EXTRA_INSTALL_AND_OPEN_DETAIL, false))) {
                // DONOTHING
            } else {
                NotificationHelper.notifyDownloadComplete(this, intent.getStringExtra("title"),
                        file);
            }
            sIsDownloading.set(false);
        }
    }

    private static boolean installAndWaitComplete(Context context, String file,
            long timeout) {
        PackageInfo packageArchiveInfo = context.getPackageManager().getPackageArchiveInfo(
                file, 0);
        if (packageArchiveInfo == null) {
            throw new AndroidRuntimeException("Can not parse the file " + file + " as apk");
        }
        final String packageName = packageArchiveInfo.packageName;

        final Object sLock = new Object();
        BroadcastReceiver receiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                synchronized (sLock) {
                    if (packageName.equalsIgnoreCase(intent.getData()
                            .getSchemeSpecificPart())) {
                        sLock.notifyAll();
                        context.getApplicationContext().unregisterReceiver(this);
                    }
                }
            }
        };
        IntentFilter filter = new IntentFilter(Intent.ACTION_PACKAGE_ADDED);
        filter.addDataScheme("package");
        context.getApplicationContext().registerReceiver(receiver, filter);
        Utils.installApk(context, file);
        synchronized (sLock) {
            try {
                sLock.wait(timeout);
            } catch (Throwable e) {
            }
        }
        try {
            context.getApplicationContext().unregisterReceiver(receiver);
        } catch (Throwable e) {
        }
        return Utils.isPackageInstalled(context, packageName);
    }
}
