/**
 * 
 */

package com.qihoo.appstore.updatelib;

import android.content.Context;
import android.content.res.Resources;
import android.content.res.Resources.NotFoundException;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.HashMap;

/**
 * @author zhangyong6
 */
class RHelper {

    private static HashMap<String, Integer> sStringResIdCache = new HashMap<String, Integer>();

    static String getString(Context context, String resName) {
        try {
            Integer resId = sStringResIdCache.get(resName);
            if (resId == null || resId <= 0) {
                resId = getStringResId(context, resName);
                sStringResIdCache.put(resName, resId);
            }
            if (resId > 0) {
                return context.getString(resId);
            } else {
                throw new NotFoundException(
                        "没有找到对应的字符串资源(" + resName + ")，请确保已经将res目录下对应的.xml文件拷贝到你的工程中");
            }
        } catch (Exception e) {
            Resources.NotFoundException androidRuntimeException = new NotFoundException(
                    "没有找到对应的字符串资源(" + resName + ")，请确保已经将res目录下对应的.xml文件拷贝到你的工程中");
            androidRuntimeException.initCause(e);
            throw androidRuntimeException;
        }
    }

    static String getString(Context context, String resName, Object... formatArgs) {
        try {
            int resId = getStringResId(context, resName);
            if (resId > 0) {
                return context.getString(resId, formatArgs);
            } else {
                throw new NotFoundException(
                        "没有找到对应的字符串资源(" + resName + ")，请确保已经将res目录下对应的.xml文件拷贝到你的工程中");
            }
        } catch (Exception e) {
            NotFoundException androidRuntimeException = new NotFoundException(
                    "没有找到对应的字符串资源(" + resName + ")，请确保已经将res目录下对应的.xml文件拷贝到你的工程中");
            androidRuntimeException.initCause(e);
            throw androidRuntimeException;
        }
    }

    static int getStringResId(Context context, String resName) {

        try {
            String packageName = context.getPackageName();
            String RClassName = packageName + ".R$string";
            Class<?> RClass = Class.forName(RClassName);
            Field field = RClass.getField(resName);
            int modifiers = field.getModifiers();
            if (field.getType().isAssignableFrom(int.class)
                    && (((modifiers & Modifier.FINAL) != 0) &&
                            ((modifiers & Modifier.PUBLIC) != 0) &&
                    ((modifiers & Modifier.STATIC) != 0))) {
                return field.getInt(RClass);
            } else {
                throw new NotFoundException(
                        "没有找到对应的字符串资源(" + resName + ")，请确保已经将res目录下对应的.xml文件拷贝到你的工程中");
            }
        } catch (Exception e) {
            NotFoundException androidRuntimeException = new NotFoundException(
                    "没有找到对应的字符串资源(" + resName + ")，请确保已经将res目录下对应的.xml文件拷贝到你的工程中");
            androidRuntimeException.initCause(e);
            throw androidRuntimeException;
        }
    }
}
