
package com.qihoo.appstore.updatelib;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.Dialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.os.Bundle;
import android.support.v4.app.DialogFragment;
import android.text.SpannableString;
import android.text.format.Formatter;
import android.text.style.ForegroundColorSpan;
import android.util.Log;

class UpdateDialogFragment extends DialogFragment implements DialogInterface.OnClickListener {

    private static final String TAG = "UpdateDialogFragment";
    private AppInfo mInfo;

    public UpdateDialogFragment() {
    }

    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        Bundle args = getArguments();
        mInfo = args.getParcelable("UpdateInfo");
        if (mInfo == null) {
            if (UpdateManager.isDebug()) {
                Log.w(TAG, "onCreateDialog UpdateInfo is null, dismiss this");
            }
            dismiss();
        }

        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());

        String title = String.format("%s（%s）", mInfo.versioName, mInfo.versionCode);
        builder.setTitle(RHelper.getString(getActivity(),
                "appstore_dialog_title_found_new_version", title));

        setAlertDialogContent(getActivity(), mInfo, builder);

        String posiText = RHelper.getString(getActivity(),
                mInfo.isPatchUpdate ? "appstore_update_patch"
                        : "appstore_update_speed");
        SpannableString sp = new SpannableString(posiText);
        sp.setSpan(new ForegroundColorSpan(Color.RED), 0, posiText.length(),
                SpannableString.SPAN_EXCLUSIVE_EXCLUSIVE);
        builder.setPositiveButton(sp, this);
        builder.setNeutralButton(RHelper.getString(getActivity(), "appstore_update"), this);
        builder.setNegativeButton(android.R.string.cancel, this);
        AlertDialog dialog = builder.create();
        return dialog;
    }

    private void setAlertDialogContent(Context context, AppInfo info,
            AlertDialog.Builder build) {
        String size = Formatter.formatFileSize(context, info.size);
        if (info.isPatchUpdate) {
            String savedSize = Formatter.formatFileSize(context, info.patchUpdateSize);
            String data = RHelper.getString(context, "appstore_dialog_message_update",
                    info.updateComment, size, savedSize);

            SpannableString sp = new SpannableString(data);
            int start = data.lastIndexOf(savedSize);
            int end = start + savedSize.length();
            if (start >= 0 && end > start && end <= data.length()) {
                sp.setSpan(new ForegroundColorSpan(Color.RED), start, end,
                        SpannableString.SPAN_EXCLUSIVE_EXCLUSIVE);
                build.setMessage(sp);
            } else {
                build.setMessage(data);
            }
        } else {
            String data = RHelper.getString(context, "appstore_dialog_message_update_patch",
                    info.updateComment, size);
            int start = data.lastIndexOf(size);
            int end = start + size.length();
            if (start >= 0 && end > start && end <= data.length()) {
                SpannableString sp = new SpannableString(data);
                sp.setSpan(new ForegroundColorSpan(Color.RED), start, end,
                        SpannableString.SPAN_EXCLUSIVE_EXCLUSIVE);
                build.setMessage(sp);
            } else {
                build.setMessage(data);
            }
        }
    }

    @Override
    public void onDismiss(DialogInterface dialog) {
        Activity activity = getActivity();
        if (activity != null) {
            activity.finish();
        }
        super.onDismiss(dialog);
    }

    @Override
    public void onClick(DialogInterface dialog, int which) {
        dismiss();
        if (which == AlertDialog.BUTTON_NEGATIVE) {
            // DO NOTHING
        } else if (which == AlertDialog.BUTTON_NEUTRAL) {
            doDirectDownload();
        } else if (which == AlertDialog.BUTTON_POSITIVE) {
            doPatchDownload(getActivity().getApplicationContext());
        }
    }

    private void doDirectDownload() {
        try {
            Activity activity = getActivity();
            String file = Utils.getUpdateFile(activity, mInfo.downloadUrl, mInfo.packageName);
            PackageManager packageManager = activity.getPackageManager();
            ApplicationInfo applicationInfo = packageManager.getApplicationInfo(mInfo.packageName,
                    0);
            String title = (String) packageManager.getApplicationLabel(applicationInfo);
            CheckUpdateService.startDownload(activity, mInfo.downloadUrl, file, title, 1, null,
                    null);
        } catch (Exception e) {
            if (UpdateManager.isDebug()) {
                Log.w(TAG, "doDirectDownload fail ", e);
            }
        }
    }

    private void doPatchDownload(Context context) {
        try {
            if (Utils.isPackageInstalled(context, Utils.APPSTORE_PKGNAME)) {
                CheckUpdateService.startAppStoreToUpdate(context, mInfo.packageName, true);
            } else {
                String url = Utils.getAppStoreDownloadUrl(context);
                String title = RHelper.getString(context, "appstore_title");
                String file = Utils.getUpdateFile(context, url, Utils.APPSTORE_PKGNAME);
                CheckUpdateService.startDownload(context, url, file, title, 3, null,
                        mInfo.packageName);
            }
        } catch (Exception e) {
            if (UpdateManager.isDebug()) {
                Log.w(TAG, "doPatchDownload fail ", e);
            }
        }
    }

    @Override
    public void onDestroy() {
        try {
            Activity activity = getActivity();
            if (activity != null) {
                activity.finish();
            }
        } catch (Exception e) {
        }
        super.onDestroy();
    }
}
